@@ -4,7 +4,7 @@
 ;; LCD Archive Entry:
 ;; edebug|Daniel LaLiberte|liberte@cs.uiuc.edu
 ;; |A source level debugger for Emacs Lisp.
-;; |$Date: 92/05/16 14:10:55 $|$Revision: 2.8 $|~/modes/edebug.el
+;; |$Date: 92/06/03 13:29:45 $|$Revision: 2.9 $|~/modes/edebug.el
 
 ;; This file will be part of GNU Emacs.
 
@@ -77,8 +77,11 @@
 ;;; Change list
 ;;; -----------
 
-;;; $Header: edebug.el,v 2.8 92/05/16 14:10:55 devin Exp $
+;;; $Header: edebug.el,v 2.9 92/06/03 13:29:45 jwz Exp $
 ;;; $Log:	edebug.el,v $
+;;; Revision 2.9  92/06/03  13:29:45  jwz
+;;; *** empty log message ***
+;;; 
 ;;; Revision 2.8  92/05/16  14:10:55  devin
 ;;; *** empty log message ***
 ;;; 
@@ -913,8 +916,8 @@ or if an error occurs, leave point after it with mark at the original point."
 	  defun-interactive
 	  (` (edebug-enter
 	      (quote (, def-name))
-	      (list (,@ (delq 'rest (delq '&optional 
-					  (copy-sequence def-args)))))
+	      (list (,@ (delq '&rest (delq '&optional 
+					   (copy-sequence def-args)))))
 	      (function 
 	       (lambda ()
 		 ;; the remainder is a list of forms
